package com.bocloud.cmp.provider.compute;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.common.enums.TencentEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.provider.TencentProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.tencentconvertor.ImageConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.tencentcloudapi.common.exception.TencentCloudSDKException;
import com.tencentcloudapi.cvm.v20170312.CvmClient;
import com.tencentcloudapi.cvm.v20170312.models.CreateImageRequest;
import com.tencentcloudapi.cvm.v20170312.models.DeleteImagesRequest;
import com.tencentcloudapi.cvm.v20170312.models.DeleteImagesResponse;
import com.tencentcloudapi.cvm.v20170312.models.DescribeImagesRequest;
import com.tencentcloudapi.cvm.v20170312.models.DescribeImagesResponse;
import com.tencentcloudapi.cvm.v20170312.models.Filter;
import com.tencentcloudapi.cvm.v20170312.models.Image;
import com.tencentcloudapi.cvm.v20170312.models.ModifyImageAttributeRequest;
import com.tencentcloudapi.cvm.v20170312.models.ModifyImageSharePermissionRequest;
import com.tencentcloudapi.cvm.v20170312.models.ModifyImageSharePermissionResponse;
import com.tencentcloudapi.cvm.v20170312.models.SyncImagesRequest;

/**
 * 镜像
 * 
 * @author ymh
 * @时间 2018年11月15日
 */
@Provider("bocloud.cmp.tencent")
public class TencentImageProvider extends TencentProvider implements ImageProvider {

    public TencentImageProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(TencentImageProvider.class);

    private BeanConvertor<ImageModel, Image> convertor = new ImageConvertor();

    @Override
    public BocloudResult list() {
        BocloudResult result = new BocloudResult(false, "查询失败！");
        DescribeImagesResponse response = null;
        // 腾讯分页查询的偏移量
        Integer offset = 0;
        DescribeImagesRequest imageRequest = new DescribeImagesRequest();
        imageRequest.setLimit(REQUEST_LIMIT);
        List<Image> images = new ArrayList<>();
        try {
            response = ((CvmClient) this.getClient(TencentEnum.CVM)).DescribeImages(imageRequest);
            images.addAll(Arrays.asList(response.getImageSet()));
            // 循环分页查询
            while (images.size() >= REQUEST_LIMIT + offset) {
                offset = offset + REQUEST_LIMIT;
                imageRequest.setOffset(offset);
                response = ((CvmClient) this.getClient(TencentEnum.CVM)).DescribeImages(imageRequest);
                images.addAll(Arrays.asList(response.getImageSet()));
            }
            result = new BocloudResult(true, convertor.convertList(images), "success");
        } catch (TencentCloudSDKException e) {
            logger.error("list images error:", e);
            result = new BocloudResult(false, e.getMessage());
        }
        return result;
    }

    @Override
    public BocloudResult detail(String imageId) {// img-oikl1tzv
        try {
            DescribeImagesRequest req = new DescribeImagesRequest();
            req.setImageIds(new String[] { imageId });
            CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
            DescribeImagesResponse resp = client.DescribeImages(req);
            ImageModel imageModel = convertor.convertModel(resp.getImageSet()[0]);
            imageModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, imageModel, "查询镜像成功");
        } catch (TencentCloudSDKException e) {
            logger.error("查询镜像失败" + e);
            return new BocloudResult(false, "获取镜像列表失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String imageId) {
        try {
            DeleteImagesRequest req = new DeleteImagesRequest();
            req.setImageIds(new String[] { imageId });
            CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
            DeleteImagesResponse resp = client.DeleteImages(req);
            return new BocloudResult(true, resp.getRequestId(), "删除成功！");
        } catch (Exception e) {
            logger.error("镜像删除失败" + e);
            return new BocloudResult(false, "删除失败！", e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(ImageModel image) {
        try {
            CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
            ModifyImageAttributeRequest req = new ModifyImageAttributeRequest();
            req.setImageId(image.getImageUuid());
            if (!StringUtils.isEmpty(image.getName())) {
                req.setImageName(image.getName());
            }
            if (!StringUtils.isEmpty(image.getRemark())) {
                req.setImageDescription(image.getRemark());
            }
            client.ModifyImageAttribute(req);
            return new BocloudResult(true, this.detail(image.getImageUuid()).getData(), "修改成功！");
        } catch (Exception e) {
            logger.error("修改删除失败" + e);
            return new BocloudResult(false, "修改失败！", e.getMessage());
        }
    }

    private Image getImageByName(String imageName, String region) {
        Image image = null;
        try {
            CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
            if (!StringUtils.isEmpty(region)) {
                client.setRegion(region);
            }
            DescribeImagesRequest req = new DescribeImagesRequest();
            Filter filter = new Filter();
            filter.setName("image-type");
            filter.setValues(new String[] { "PRIVATE_IMAGE" });
            req.setFilters(new Filter[] { filter });
            Integer flag = 60; // 5 min
            while (flag > 0) {
                DescribeImagesResponse resp = client.DescribeImages(req);
                Image[] imageSet = resp.getImageSet();
                for (Image im : imageSet) {
                    if (imageName.equals(im.getImageName()) && "NORMAL".equals(im.getImageState())) {
                        image = im;
                        flag = -1;
                        break;
                    }
                }
                Thread.sleep(10000);
                flag--;
            }
        } catch (Exception e) {
            logger.error("查询镜像详情失败" + e);
        }
        return image;
    }

    @Override
    public BocloudResult create(ImageModel imageModel) {
        try {
            CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
            CreateImageRequest req = new CreateImageRequest();
            req.setInstanceId(imageModel.getInstanceId());
            req.setImageName(imageModel.getName());
            req.setImageDescription(imageModel.getRemark());
            client.CreateImage(req);
            Image imageByName = getImageByName(imageModel.getName(), null);
            if (null != imageByName) {
                ImageModel model = convertor.convertModel(imageByName);
                model.setRegionId(this.getRegionId());
                return new BocloudResult(true, model, "创建镜像成功！");
            } else {
                return new BocloudResult(false, "创建镜像超时", "创建镜像超时");
            }
        } catch (Exception e) {
            logger.error("镜像创建失败 ： " + e);
            return new BocloudResult(false, "镜像创建失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult shareImage(ImageModel imageModel) {
        ModifyImageSharePermissionRequest request = new ModifyImageSharePermissionRequest();
        request.setImageId(imageModel.getImageUuid());
        request.setAccountIds(imageModel.getAccountIds());
        request.setPermission(imageModel.getPermission());
        ModifyImageSharePermissionResponse response;
        try {
            response = ((CvmClient) this.getClient(TencentEnum.CVM)).ModifyImageSharePermission(request);
            return new BocloudResult(true, response.getRequestId(), "镜像分享成功");
        } catch (TencentCloudSDKException e) {
            logger.error("image ShartImage [{}] error:", e);
            return new BocloudResult(false, "镜像分享成功失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult syncImages(ImageModel imageModel) {
        SyncImagesRequest request = new SyncImagesRequest();
        request.setDestinationRegions(new String[] { imageModel.getRegionId() });
        request.setImageIds(new String[] { imageModel.getImageUuid() });
        try {
            ((CvmClient) this.getClient(TencentEnum.CVM)).SyncImages(request);
            Image image = this.getImageByName(imageModel.getName(), imageModel.getRegionId());
            return new BocloudResult(true, convertor.convertModel(image), "镜像跨域复制成功");
        } catch (TencentCloudSDKException e) {
            logger.error("image SyncImage [{}] error:", e);
            return new BocloudResult(false, "镜像跨域复制失败", e.getMessage());
        }
    }

}
