package com.bocloud.cmp.provider.loadbalance;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.common.enums.TencentEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.HealthMonitorModel;
import com.bocloud.cmp.model.LbListenerModel;
import com.bocloud.cmp.model.LbMemberModel;
import com.bocloud.cmp.model.LbModel;
import com.bocloud.cmp.model.LbPoolModel;
import com.bocloud.cmp.model.VipModel;
import com.bocloud.cmp.provider.TencentProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.tencentconvertor.ListenerConvertor;
import com.bocloud.cmp.tencentconvertor.LoadBalanceConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.tencentcloudapi.clb.v20180317.ClbClient;
import com.tencentcloudapi.clb.v20180317.models.CreateListenerRequest;
import com.tencentcloudapi.clb.v20180317.models.CreateListenerResponse;
import com.tencentcloudapi.clb.v20180317.models.CreateLoadBalancerRequest;
import com.tencentcloudapi.clb.v20180317.models.CreateLoadBalancerResponse;
import com.tencentcloudapi.clb.v20180317.models.DeleteLoadBalancerRequest;
import com.tencentcloudapi.clb.v20180317.models.DeleteLoadBalancerResponse;
import com.tencentcloudapi.clb.v20180317.models.DescribeListenersRequest;
import com.tencentcloudapi.clb.v20180317.models.DescribeListenersResponse;
import com.tencentcloudapi.clb.v20180317.models.DescribeLoadBalancersRequest;
import com.tencentcloudapi.clb.v20180317.models.DescribeLoadBalancersResponse;
import com.tencentcloudapi.clb.v20180317.models.Listener;
import com.tencentcloudapi.clb.v20180317.models.LoadBalancer;
import com.tencentcloudapi.common.exception.TencentCloudSDKException;

/**
 * tencent负载均衡
 */
@Provider("bocloud.cmp.tencent")
public class TencentLoadBalanceProvider extends TencentProvider implements LoadBalanceProvider {

    private static Logger logger = LoggerFactory.getLogger(TencentLoadBalanceProvider.class);

    private BeanConvertor<LbModel, LoadBalancer> convertor = new LoadBalanceConvertor();

    private BeanConvertor<LbListenerModel, Listener> convertorListener = new ListenerConvertor();

    public TencentLoadBalanceProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            ClbClient clbClient = (ClbClient) this.getClient(TencentEnum.CLB);
            DescribeLoadBalancersRequest req = new DescribeLoadBalancersRequest();
            DescribeLoadBalancersResponse resp = clbClient.DescribeLoadBalancers(req);
            LoadBalancer[] loadBalancerSet = resp.getLoadBalancerSet();
            List<LbModel> convertList = convertor.convertList(Arrays.asList(loadBalancerSet));
            for (LbModel lbModel : convertList) {
                lbModel.setRegionId(this.getRegionId());
                lbModel.setModels(listenerModelList(lbModel.getLbUuid()));
            }
            return new BocloudResult(true, convertList, "查询负载均衡实例成功");
        } catch (TencentCloudSDKException e) {
            return new BocloudResult(false, "查询负载均衡实例失败", e.getMessage());
        }
    }

    private List<LbListenerModel> listenerModelList(String lbId) {
        List<LbListenerModel> models = new ArrayList<>();
        BocloudResult listListener = this.listListener(lbId);
        if (listListener.isSuccess()) {
            models = JSON.parseArray(JSON.toJSONString(listListener.getData()), LbListenerModel.class);
        }
        return models;
    }

    @Override
    public BocloudResult create(LbModel model) {
        try {
            ClbClient clbClient = (ClbClient) this.getClient(TencentEnum.CLB);
            CreateLoadBalancerRequest req = new CreateLoadBalancerRequest();
            // 负载均衡实例的网络类型： OPEN：公网属性， INTERNAL：内网属性。
            String loadBalancerType = model.getLoadBalancerType();
            req.setLoadBalancerType(loadBalancerType);
            // 负载均衡实例。1：应用型，0：传统型，默认为应用型负载均衡实例。
            req.setForward(1);
            req.setLoadBalancerName(model.getName());
            req.setVpcId(model.getVpcUuid());
            if ("INTERNAL".equals(loadBalancerType)) { // OPEN 公网 INTERNAL 内网
                req.setSubnetId(model.getSubnetUuid());
            }
            CreateLoadBalancerResponse createLoadBalancer = clbClient.CreateLoadBalancer(req);
            String[] loadBalancerIds = createLoadBalancer.getLoadBalancerIds();
            LbModel availableLoadBalance = this.getAvailableLoadBalance(loadBalancerIds[0]);
            availableLoadBalance.setRegionId(this.getRegionId());
            return new BocloudResult(true, availableLoadBalance, "创建负载均衡成功");
        } catch (TencentCloudSDKException e) {
            logger.error("创建负载均衡失败" + e);
            return new BocloudResult(false, "创建负载均衡失败", e.getMessage());
        }
    }

    private LbModel getAvailableLoadBalance(String lbId) {
        DescribeLoadBalancersRequest request = new DescribeLoadBalancersRequest();
        request.setLoadBalancerIds(new String[] { lbId });
        LbModel lbModel = new LbModel();
        try {
            Boolean flag = true;
            Long startTime = System.currentTimeMillis();
            while (flag) {
                try {
                    Thread.sleep(3000);
                } catch (InterruptedException e) {
                    logger.error("Get error message:", e);
                    Thread.currentThread().interrupt();
                }
                DescribeLoadBalancersResponse response = ((ClbClient) this.getClient(TencentEnum.CLB))
                        .DescribeLoadBalancers(request);
                List<LoadBalancer> loadBalancers = Arrays.asList(response.getLoadBalancerSet());
                if (ListTool.isEmpty(loadBalancers)) {
                    flag = false;
                    break;
                }
                if (1 == (loadBalancers.get(0).getStatus())) {
                    lbModel = convertor.convertModel(loadBalancers.get(0));
                    flag = false;
                    break;
                }
                Long endTime = System.currentTimeMillis();
                // 等待60s
                if (endTime - startTime > 3 * 60 * 1000) {
                    lbModel = convertor.convertModel(loadBalancers.get(0));
                    flag = false;
                    break;
                }
            }
        } catch (TencentCloudSDKException e) {
            logger.error("get loadbalancer error:", e);
        }
        return lbModel;

    }

    @Override
    public BocloudResult update(LbModel lb) {
        return null;
    }

    @Override
    public BocloudResult remove(String lbId) {
        try {
            ClbClient clbClient = (ClbClient) this.getClient(TencentEnum.CLB);
            DeleteLoadBalancerRequest req = new DeleteLoadBalancerRequest();
            req.setLoadBalancerIds(new String[] { lbId });
            DeleteLoadBalancerResponse resp = clbClient.DeleteLoadBalancer(req);
            return new BocloudResult(true, resp.getRequestId(), "删除负载均衡实例成功");
        } catch (TencentCloudSDKException e) {
            return new BocloudResult(false, "删除负载均衡实例失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String lbId) {
        try {
            ClbClient clbClient = (ClbClient) this.getClient(TencentEnum.CLB);
            DescribeLoadBalancersRequest req = new DescribeLoadBalancersRequest();
            req.setLoadBalancerIds(new String[] { lbId });
            DescribeLoadBalancersResponse resp = clbClient.DescribeLoadBalancers(req);
            LoadBalancer[] loadBalancerSet = resp.getLoadBalancerSet();
            LbModel lbModel = convertor.convertModel(loadBalancerSet[0]);
            lbModel.setRegionId(this.getRegionId());
            lbModel.setModels(listenerModelList(lbModel.getLbUuid()));
            return new BocloudResult(true, lbModel, "查询负载均衡详情成功");
        } catch (TencentCloudSDKException e) {
            return new BocloudResult(false, "查询负载均衡详情失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult listVip() {
        return null;
    }

    @Override
    public BocloudResult createVip(VipModel vModel) {
        return null;
    }

    @Override
    public BocloudResult updateVip(VipModel vModel) {
        return null;
    }

    @Override
    public BocloudResult removeVip(String vipId) {
        return null;
    }

    @Override
    public BocloudResult detailVip(String vipId) {
        return null;
    }

    @Override
    public BocloudResult listListener() {
        return null;
    }

    @Override
    public BocloudResult listListener(String lbId) {
        try {
            ClbClient client = (ClbClient) this.getClient(TencentEnum.CLB);
            DescribeListenersRequest req = new DescribeListenersRequest();
            req.setLoadBalancerId(lbId);
            DescribeListenersResponse resp = client.DescribeListeners(req);
            Listener[] listeners = resp.getListeners();
            return new BocloudResult(true, convertorListener.convertList(Arrays.asList(listeners)), "查询成功");
        } catch (Exception e) {
            logger.error("查询失败" + e);
            return new BocloudResult(false, "查询失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult createListener(LbListenerModel lbListenerModel) {
        try {
            ClbClient client = (ClbClient) this.getClient(TencentEnum.CLB);
            CreateListenerRequest req = new CreateListenerRequest();
            req.setListenerNames(new String[] { lbListenerModel.getName() });
            req.setLoadBalancerId(lbListenerModel.getLbUuid());
            req.setProtocol(lbListenerModel.getProtocol());
            req.setPorts(new Integer[] { lbListenerModel.getPort() });
            CreateListenerResponse resp = client.CreateListener(req);
            String[] listenerIds = resp.getListenerIds();
            return new BocloudResult(true, listenerIds, "创建监听器成功");
        } catch (Exception e) {
            logger.error("创建监听器失败" + e);
            return new BocloudResult(false, "创建监听器失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult updateListener(LbListenerModel lbListenerModel) {
        return null;
    }

    @Override
    public BocloudResult removeListener(String listenerId) {
        return null;
    }

    @Override
    public BocloudResult detailListener(String listenerId) {
        return null;
    }

    @Override
    public BocloudResult listHealMonitor() {
        return null;
    }

    @Override
    public BocloudResult createHealMonitor(HealthMonitorModel hmModel) {
        return null;
    }

    @Override
    public BocloudResult updateHealMonitor(HealthMonitorModel hmModel) {
        return null;
    }

    @Override
    public BocloudResult removeHealMonitor(String hmId) {
        return null;
    }

    @Override
    public BocloudResult detailHealMonitor(String hmId) {
        return null;
    }

    @Override
    public BocloudResult listMember(String poolId) {
        return null;
    }

    @Override
    public BocloudResult createMember(LbMemberModel mbModel) {
        return null;
    }

    @Override
    public BocloudResult updateMember(LbMemberModel mbModel) {
        return null;
    }

    @Override
    public BocloudResult removeMember(String poolId, String mbId) {
        return null;
    }

    @Override
    public BocloudResult detailMember(String poolId, String mbId) {
        return null;
    }

    @Override
    public BocloudResult listPool() {
        return null;
    }

    @Override
    public BocloudResult createPool(LbPoolModel poolModel) {
        return null;
    }

    @Override
    public BocloudResult removePool(String poolId) {
        return null;
    }

    @Override
    public BocloudResult updatePool(LbPoolModel poolModel) {
        return null;
    }

    @Override
    public BocloudResult detailPool(String poolId) {
        return null;
    }

}
