package com.bocloud.cmp.provider.network;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

import com.tencentcloudapi.vpc.v20170312.models.*;
import org.apache.commons.collections.ListUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.bocloud.cmp.common.enums.TencentEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.provider.TencentProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.tencentconvertor.SecurityGroupConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.tencentcloudapi.vpc.v20170312.VpcClient;

/**
 * @author ymh
 * @时间 2018年11月15日
 */
@Provider("bocloud.cmp.tencent")
public class TencentSecuGroupProvider extends TencentProvider implements SecurityGroupProvider {

    private static Logger logger = LoggerFactory.getLogger(TencentSecuGroupProvider.class);

    private BeanConvertor<SecurityGroupModel, SecurityGroup> convertor = new SecurityGroupConvertor();

    public TencentSecuGroupProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            DescribeSecurityGroupsRequest req = new DescribeSecurityGroupsRequest();
            Filter filter = new Filter();
            filter.setName("project-id");
            filter.setValues(new String[]{this.getProjectId()});
            Filter[] filters = new Filter[]{filter};
            req.setFilters(filters);
            VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
            DescribeSecurityGroupsResponse resp = client.DescribeSecurityGroups(req);
            SecurityGroup[] groupSet = resp.getSecurityGroupSet();

            List<SecurityGroupModel> securityGroupModels = convertor.convertList(Arrays.asList(groupSet));
            Map<String, SecurityGroupModel> groupModelMap = securityGroupModels.stream().collect(Collectors.toMap(SecurityGroupModel::getGroupUuid, Function.identity()));

            DescribeSecurityGroupAssociationStatisticsRequest statisticsReq = new DescribeSecurityGroupAssociationStatisticsRequest();
            statisticsReq.setSecurityGroupIds(groupModelMap.keySet().toArray(new String[groupModelMap.keySet().size()]));
            DescribeSecurityGroupAssociationStatisticsResponse statisticsResponse = client.DescribeSecurityGroupAssociationStatistics(statisticsReq);
            SecurityGroupAssociationStatistics[] associationStatistics = statisticsResponse.getSecurityGroupAssociationStatisticsSet();
            Map<String, SecurityGroupAssociationStatistics> statisticsMap = Arrays.asList(associationStatistics).stream().collect(Collectors.toMap(SecurityGroupAssociationStatistics::getSecurityGroupId, Function.identity()));

            for (SecurityGroupModel securityGroupModel : securityGroupModels) {
                SecurityGroupAssociationStatistics groupAssociationStatistics = statisticsMap.get(securityGroupModel.getGroupUuid());
                Integer sum = 0;
                for (InstanceStatistic instanceStatistic : groupAssociationStatistics.getInstanceStatistics()) {
                    if (instanceStatistic.getInstanceCount() != null && instanceStatistic.getInstanceCount() > 0) {
                        sum += instanceStatistic.getInstanceCount();
                    }
                }
                securityGroupModel.setInstanceCount(sum);
            }
            return new BocloudResult(true, securityGroupModels, "查询成功");
        } catch (Exception e) {
            logger.error("查询失败 ： " + e);
            return new BocloudResult(false, "查询失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult create(SecurityGroupModel group) {
        try {
            VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
            CreateSecurityGroupRequest req = new CreateSecurityGroupRequest();
            req.setGroupName(group.getName());
            String remark = group.getRemark();
            if (null == remark || "".equals(remark)) {
                remark = "";
            }
            req.setGroupDescription(remark);
            String proId = this.getProjectId();
            if (null != proId && !"".equals(proId)) {
                req.setProjectId(proId);
            }
            CreateSecurityGroupResponse resp = client.CreateSecurityGroup(req);
            SecurityGroup securityGroup = resp.getSecurityGroup();
            SecurityGroupModel groupModel = convertor.convertModel(securityGroup);
            groupModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, groupModel, "创建成功");
        } catch (Exception e) {
            logger.error("创建失败 ： " + e);
            return new BocloudResult(false, "创建失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String sgId) {
        try {
            VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
            DeleteSecurityGroupRequest req = new DeleteSecurityGroupRequest();
            req.setSecurityGroupId(sgId);
            DeleteSecurityGroupResponse resp = client.DeleteSecurityGroup(req);
            return new BocloudResult(true, resp.getRequestId(), "删除成功");
        } catch (Exception e) {
            logger.error("删除失败 ： " + e);
            return new BocloudResult(false, "删除失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String sgId) {
        try {
            DescribeSecurityGroupsRequest req = new DescribeSecurityGroupsRequest();
            VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
            req.setSecurityGroupIds(new String[]{sgId});
            DescribeSecurityGroupsResponse resp = client.DescribeSecurityGroups(req);
            SecurityGroup[] securityGroupSet = resp.getSecurityGroupSet();
            SecurityGroup group = null;
            if (null != securityGroupSet && securityGroupSet.length > 0) {
                group = securityGroupSet[0];
            }
            return new BocloudResult(true, convertor.convertModel(group), "查询成功");
        } catch (Exception e) {
            logger.error("查询失败 ： " + e);
            return new BocloudResult(false, "查询失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(SecurityGroupModel model) {
        try {
            VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
            ModifySecurityGroupAttributeRequest req = new ModifySecurityGroupAttributeRequest();
            req.setSecurityGroupId(model.getGroupUuid());
            if (null != model.getName() && !"".equals(model.getName())) {
                req.setGroupName(model.getName());
            }
            if (null != model.getRemark() && !"".equals(model.getRemark())) {
                req.setGroupDescription(model.getRemark());
            }
            ModifySecurityGroupAttributeResponse resp = client.ModifySecurityGroupAttribute(req);
            return new BocloudResult(true, resp.getRequestId(), "修改成功");
        } catch (Exception e) {
            logger.error("修改失败 ： " + e);
            return new BocloudResult(false, "修改失败", e.getMessage());
        }
    }

}
