package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.common.enums.TencentEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.TencentProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.tencentcloudapi.vpc.v20170312.VpcClient;
import com.tencentcloudapi.vpc.v20170312.models.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * 安全组规则操作类
 *
 * @author ymh
 * @时间 2018年11月15日
 */
@Provider("bocloud.cmp.tencent")
public class TencentSecuGroupRuleProvider extends TencentProvider implements SecurityGroupRuleProvider {

	private static Logger logger = LoggerFactory.getLogger(TencentSecuGroupRuleProvider.class);

	private static final String EGRESS = "egress";
	private static final String INGRESS = "ingress";
	private static final String ALL = "ALL";

	public TencentSecuGroupRuleProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult create(SecurityGroupRuleModel model) {
		try {
			VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
			CreateSecurityGroupPoliciesRequest req = new CreateSecurityGroupPoliciesRequest();
			req.setSecurityGroupId(model.getGroupUuid());
			SecurityGroupPolicySet securityGroupPolicySet = new SecurityGroupPolicySet();
			List<SecurityGroupPolicy> policyList = new ArrayList<SecurityGroupPolicy>();
			SecurityGroupPolicy policy = new SecurityGroupPolicy();
			String protocol = model.getProtocol();
			policy.setProtocol(protocol);
			if (!"icmp".equalsIgnoreCase(protocol)) {
				Integer portMax = model.getPortMax();
				Integer portMin = model.getPortMin();
				if (null == portMax || null == portMin) {
					return new BocloudResult(false, "参数必要参数缺失！");
				}
				if (portMax.equals(portMin)) {
					policy.setPort(portMax + "");
				} else {
					policy.setPort(portMin + "-" + portMax);
				}
			}
			policy.setCidrBlock(model.getRemoteIpPrefix());
			String action = model.getAction();
			if (null == action || "".equalsIgnoreCase(action)) {
				action = "ACCEPT"; // ACCEPT允许 DROP阻止
			}
			policy.setAction(action);
			policyList.add(policy);
			SecurityGroupPolicy[] policyArr = new SecurityGroupPolicy[policyList.size()];
			String direction = model.getDirection();
			if (EGRESS.equals(direction)) {
				securityGroupPolicySet.setEgress(policyList.toArray(policyArr));
			} else if (INGRESS.equals(direction)) {
				securityGroupPolicySet.setIngress(policyList.toArray(policyArr));
			}
			req.setSecurityGroupPolicySet(securityGroupPolicySet);
			client.CreateSecurityGroupPolicies(req);
			model.setAction(model.getAction() == null ? "ACCEPT" : model.getAction().toUpperCase());
			return new BocloudResult(true, model, "添加成功");
		} catch (Exception e) {
			logger.error("创建失败 ： " + e);
			return new BocloudResult(false, "创建失败", e.getMessage());
		}
	}

	private void convertRules(SecurityGroupPolicy securityGroupPolicy, SecurityGroupRuleModel model) {
		try {
			model.setAction(securityGroupPolicy.getAction());
			model.setGroupUuid(securityGroupPolicy.getSecurityGroupId());
			String protocol = securityGroupPolicy.getProtocol();
			if (null != protocol && !"".equals(protocol)) {
				if (!ALL.equals(protocol)) {
					model.setProtocol(protocol.toUpperCase());
				}
			}
			model.setRemark(securityGroupPolicy.getPolicyDescription());
			model.setRemoteIpPrefix(securityGroupPolicy.getCidrBlock());
			if (!ALL.equalsIgnoreCase(securityGroupPolicy.getPort())) {
				String port = securityGroupPolicy.getPort();
				if (null != port && port.contains(",")) {
					// 目前不支持 不连续的端口
				} else if (null != port && port.contains("-")) {
					String[] split = port.split("-");
					model.setPortMin(Integer.parseInt(split[0]));
					model.setPortMax(Integer.parseInt(split[1]));
				} else {
					model.setPortMin(Integer.parseInt(port));
					model.setPortMax(Integer.parseInt(port));
				}
			}
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
	}

	public List<SecurityGroupRuleModel> convertModel(SecurityGroupPolicySet info) {
		List<SecurityGroupRuleModel> ruleModels = new ArrayList<SecurityGroupRuleModel>();
		SecurityGroupPolicy[] egress = info.getEgress();
		SecurityGroupRuleModel model = null;
		for (SecurityGroupPolicy securityGroupPolicy : egress) {
			model = new SecurityGroupRuleModel();
			convertRules(securityGroupPolicy, model);
			model.setDirection(EGRESS);
			ruleModels.add(model);
		}
		SecurityGroupPolicy[] ingress = info.getIngress();
		for (SecurityGroupPolicy securityGroupPolicy : ingress) {
			model = new SecurityGroupRuleModel();
			convertRules(securityGroupPolicy, model);
			model.setDirection(INGRESS);
			ruleModels.add(model);
		}
		return ruleModels;
	}

	@Override
	public BocloudResult list(String secuId) {
		VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
		DescribeSecurityGroupPoliciesRequest req = new DescribeSecurityGroupPoliciesRequest();
		req.setSecurityGroupId(secuId);
		try {
			DescribeSecurityGroupPoliciesResponse resp = client.DescribeSecurityGroupPolicies(req);
			SecurityGroupPolicySet securityGroupPolicySet = resp.getSecurityGroupPolicySet();
			return new BocloudResult(true, this.convertModel(securityGroupPolicySet), "查询成功！ ");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(true, "查询失败！ ");
		}
	}

	@Override
	public BocloudResult remove(String ruleId) {
		return null;
	}

	@Override
	public BocloudResult remove(SecurityGroupRuleModel ruleModel) {
		try {
			VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
			DeleteSecurityGroupPoliciesRequest req = new DeleteSecurityGroupPoliciesRequest();
			req.setSecurityGroupId(ruleModel.getGroupUuid());
			SecurityGroupPolicy policy = new SecurityGroupPolicy();
			DescribeSecurityGroupPoliciesRequest groupReq = new DescribeSecurityGroupPoliciesRequest();
			groupReq.setSecurityGroupId(ruleModel.getGroupUuid());
			DescribeSecurityGroupPoliciesResponse groupResp = client.DescribeSecurityGroupPolicies(groupReq);
			SecurityGroupPolicySet securityGroupPolicySet = groupResp.getSecurityGroupPolicySet();

			SecurityGroupPolicy[] gress = null;
			if (INGRESS.equalsIgnoreCase(ruleModel.getDirection())) {
				gress = securityGroupPolicySet.getIngress();
			} else if (EGRESS.equalsIgnoreCase(ruleModel.getDirection())) {
				gress = securityGroupPolicySet.getEgress();
			}
			String portModel = "";
			String currentProtocol = ruleModel.getProtocol();
			Integer portMax = ruleModel.getPortMax();
			Integer portMin = ruleModel.getPortMin();
			if (!"icmp".equalsIgnoreCase(currentProtocol)) {
				if (null != portMax && portMax.equals(portMin)) {
					portModel = String.valueOf(portMax);
				} else {
					portModel = portMin + "-" + portMax;
				}
			}
			if (null == portMax && null == portMin) {
				portModel = ALL;
			}
			// 获取安全组规则下标
			Integer policyIndex = this.getRulePolicyIndex(gress, portModel, ruleModel);
			if (null == policyIndex) {
				return new BocloudResult(false, "删除失败: 系统忙请稍后再试！");
			}
			policy.setPolicyIndex(policyIndex);
			SecurityGroupPolicy[] groupPolicy = new SecurityGroupPolicy[] { policy };
			if (INGRESS.equalsIgnoreCase(ruleModel.getDirection())) {
				securityGroupPolicySet.setIngress(groupPolicy);
				securityGroupPolicySet.setEgress(null);
			} else if (EGRESS.equalsIgnoreCase(ruleModel.getDirection())) {
				securityGroupPolicySet.setEgress(groupPolicy);
				securityGroupPolicySet.setIngress(null);
			}
			req.setSecurityGroupPolicySet(securityGroupPolicySet);
			DeleteSecurityGroupPoliciesResponse resp = client.DeleteSecurityGroupPolicies(req);
			return new BocloudResult(true, resp.getRequestId(), "删除成功！");
		} catch (Exception e) {
			logger.error(e.getMessage());
			return new BocloudResult(false, "删除失败 : " + e.getMessage());
		}
	}

	private Integer getRulePolicyIndex(SecurityGroupPolicy[] gress, String portModel,
			SecurityGroupRuleModel ruleModel) {
		Integer policyIndex = null;
		try {
			for (SecurityGroupPolicy securityGroupPolicy : gress) {
				String action = securityGroupPolicy.getAction();
				String cidrBlock = securityGroupPolicy.getCidrBlock();
				String port = securityGroupPolicy.getPort();
				String protocol = securityGroupPolicy.getProtocol();
				if (action.equalsIgnoreCase(ruleModel.getAction()) && cidrBlock.equalsIgnoreCase(ruleModel.getRemoteIpPrefix())
						&& port.equalsIgnoreCase(portModel) && protocol.equalsIgnoreCase(ruleModel.getProtocol())) {
					policyIndex = securityGroupPolicy.getPolicyIndex();
					break;
				}
			}
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
		return policyIndex;
	}

}
