package com.bocloud.cmp.provider.network;

import java.util.Arrays;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.common.enums.TencentEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.VpcModel;
import com.bocloud.cmp.provider.TencentProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.tencentconvertor.VpcConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.tencentcloudapi.vpc.v20170312.VpcClient;
import com.tencentcloudapi.vpc.v20170312.models.CreateVpcRequest;
import com.tencentcloudapi.vpc.v20170312.models.CreateVpcResponse;
import com.tencentcloudapi.vpc.v20170312.models.DeleteVpcRequest;
import com.tencentcloudapi.vpc.v20170312.models.DeleteVpcResponse;
import com.tencentcloudapi.vpc.v20170312.models.DescribeVpcsRequest;
import com.tencentcloudapi.vpc.v20170312.models.DescribeVpcsResponse;
import com.tencentcloudapi.vpc.v20170312.models.ModifyVpcAttributeRequest;
import com.tencentcloudapi.vpc.v20170312.models.ModifyVpcAttributeResponse;
import com.tencentcloudapi.vpc.v20170312.models.Vpc;

/**
 * @author : guopeng create at: 2018/11/15 11:40 AM
 * @description:
 */
@Provider("bocloud.cmp.tencent")
public class TencentVpcProvider extends TencentProvider implements VpcProvider {

	private static Logger logger = LoggerFactory.getLogger(TencentVpcProvider.class);

	private BeanConvertor<VpcModel, Vpc> convertor = new VpcConvertor();

	public TencentVpcProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult create(VpcModel model) {
		try {
			VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
			CreateVpcRequest req = new CreateVpcRequest();
			req.setVpcName(model.getName());
			req.setCidrBlock(model.getCidr());
			CreateVpcResponse resp = client.CreateVpc(req);
			Object data = this.getVpcDetail(resp.getVpc().getVpcId(), this.getRegionId()).getData();
			VpcModel convertModel = JSON.parseObject(JSON.toJSONString(data), VpcModel.class);
			convertModel.setRegionId(this.getRegionId());
			return new BocloudResult(true, convertModel, "创建成功");
		} catch (Exception e) {
			logger.error("创建失败 ： " + e);
			String message = e.getMessage();
			if(message.contains("LimitExceeded")) {
				message = "当前区域`Vpc`数量达到上限";
			}
			return new BocloudResult(false, "创建失败", message);
		}
	}
	
	public BocloudResult getVpcDetail(String vpcId, String regionId) {
		BocloudResult result = null;
		try {
			Boolean flag = true;
			Long startTime = System.currentTimeMillis();
			while (flag) {
				Thread.sleep(100);
				VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
				client.setRegion(regionId);
				DescribeVpcsRequest req = new DescribeVpcsRequest();
				req.setVpcIds(new String[] { vpcId });
				DescribeVpcsResponse resp = client.DescribeVpcs(req);
				Vpc[] vpcSet = resp.getVpcSet();

				if (vpcSet == null || vpcSet.length <= 0) {
					flag = false;
					result = new BocloudResult(false, "失败");
					break;
				}
				Vpc vpc = vpcSet[0];
				if (null != vpc.getCreatedTime() && !vpc.getCreatedTime().equals("0000-00-00 00:00:00")) {
					flag = false;
					result = this.detail(vpcId, regionId);
					break;
				}
				Long endTime = System.currentTimeMillis();
				// 等待60s
				if (endTime - startTime > 3 * 60 * 1000) {
					flag = false;
					result = this.detail(vpcId, regionId);
					break;
				}
			}
			return result;
		} catch (Exception e) {
			logger.error("查询状态失败： " + e);
			return new BocloudResult(false, "查询状态失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String vpcId) {
		try {
			VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
			DeleteVpcRequest req = new DeleteVpcRequest();
			req.setVpcId(vpcId);
			DeleteVpcResponse resp = client.DeleteVpc(req);
			return new BocloudResult(true, resp.getRequestId(), "删除成功");
		} catch (Exception e) {
			logger.error("删除失败 ： " + e);
			return new BocloudResult(false, "删除失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult list() {
		try {
			VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
			DescribeVpcsRequest req = new DescribeVpcsRequest();
			DescribeVpcsResponse resp = client.DescribeVpcs(req);
			Vpc[] vpcSet = resp.getVpcSet();
			return new BocloudResult(true, convertor.convertList(Arrays.asList(vpcSet)), "查询成功");
		} catch (Exception e) {
			logger.error("查询列表失败 ： " + e);
			return new BocloudResult(false, "查询列表失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String vpcId, String regionId) {
		try {
			VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
			if (null != regionId && !"".equals(regionId)) {
				client.setRegion(regionId);
			}
			DescribeVpcsRequest req = new DescribeVpcsRequest();
			req.setVpcIds(new String[] { vpcId });
			DescribeVpcsResponse resp = client.DescribeVpcs(req);
			Vpc[] vpcSet = resp.getVpcSet();
			return new BocloudResult(true, convertor.convertModel(vpcSet[0]), "查询成功");
		} catch (Exception e) {
			logger.error("查询列表失败 ： " + e);
			return new BocloudResult(false, "查询列表失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult update(VpcModel model) {
		try {
			VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
			if (null != model.getRegionId() && !"".equals(model.getRegionId())) {
				client.setRegion(model.getRegionId());
			}
			ModifyVpcAttributeRequest req = new ModifyVpcAttributeRequest();
			req.setVpcId(model.getVpcId());
			req.setDomainName(model.getName());
			ModifyVpcAttributeResponse resp = client.ModifyVpcAttribute(req);
			return new BocloudResult(true, resp.getRequestId(), "修改成功");
		} catch (Exception e) {
			logger.error("修改失败 ： " + e);
			return new BocloudResult(false, "修改失败", e.getMessage());
		}
	}

}
