package com.bocloud.cmp.provider.security;

import java.util.Arrays;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.common.enums.TencentEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.KeypairModel;
import com.bocloud.cmp.provider.TencentProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.tencentconvertor.KeypairConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.tencentcloudapi.common.exception.TencentCloudSDKException;
import com.tencentcloudapi.cvm.v20170312.CvmClient;
import com.tencentcloudapi.cvm.v20170312.models.CreateKeyPairRequest;
import com.tencentcloudapi.cvm.v20170312.models.CreateKeyPairResponse;
import com.tencentcloudapi.cvm.v20170312.models.DeleteKeyPairsRequest;
import com.tencentcloudapi.cvm.v20170312.models.DeleteKeyPairsResponse;
import com.tencentcloudapi.cvm.v20170312.models.DescribeKeyPairsRequest;
import com.tencentcloudapi.cvm.v20170312.models.DescribeKeyPairsResponse;
import com.tencentcloudapi.cvm.v20170312.models.ImportKeyPairRequest;
import com.tencentcloudapi.cvm.v20170312.models.ImportKeyPairResponse;
import com.tencentcloudapi.cvm.v20170312.models.KeyPair;
import com.tencentcloudapi.cvm.v20170312.models.ModifyKeyPairAttributeRequest;
import com.tencentcloudapi.cvm.v20170312.models.ModifyKeyPairAttributeResponse;

/**
 * 密钥操作类
 * 
 * @author ymh
 * @时间 2018年11月15日
 */
@Provider("bocloud.cmp.tencent")
public class TencentKeypairProvider extends TencentProvider implements KeypairProvider {

	private static Logger logger = LoggerFactory.getLogger(TencentKeypairProvider.class);

	private BeanConvertor<KeypairModel, KeyPair> convertor = new KeypairConvertor();

	public TencentKeypairProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list() {
		DescribeKeyPairsRequest request = new DescribeKeyPairsRequest();
		DescribeKeyPairsResponse response;
		try {
			CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
			response = client.DescribeKeyPairs(request);
			KeyPair[] keyPairs = response.getKeyPairSet();
			return new BocloudResult(true, convertor.convertList(Arrays.asList(keyPairs)), "查询密钥对列表完成");
		} catch (TencentCloudSDKException e) {
			logger.error("查询密钥对列表 error:", e);
			return new BocloudResult(false, "查询密钥对列表失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String keypairName) {
		DescribeKeyPairsRequest request = new DescribeKeyPairsRequest();
		DescribeKeyPairsResponse response;
		try {
			CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
			request.setKeyIds(new String[] { keypairName });
			response = client.DescribeKeyPairs(request);
			KeyPair[] keyPairs = response.getKeyPairSet();
			KeyPair key = null;
			if (null != keyPairs && keyPairs.length > 0) {
				key = keyPairs[0];
			}
			return new BocloudResult(true, convertor.convertModel(key), "查询详情");
		} catch (TencentCloudSDKException e) {
			logger.error("查询详情 error:", e);
			return new BocloudResult(false, "查询详情失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult create(String keypairName, String publicKey) {
		try {
			CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
			if (!StringUtils.isEmpty(publicKey)) {
				ImportKeyPairRequest request = new ImportKeyPairRequest();
				request.setKeyName(keypairName);
				String proId = this.getProjectId();
				if (null != proId && !"".equals(proId)) {
					request.setProjectId(Integer.parseInt(this.getProjectId()));
				}
				request.setPublicKey(publicKey);
				ImportKeyPairResponse importKeyPair = client.ImportKeyPair(request);
				String keyId = importKeyPair.getKeyId();
				return this.detail(keyId);
			} else {
				CreateKeyPairRequest req = new CreateKeyPairRequest();
				req.setKeyName(keypairName);
				String proId = this.getProjectId();
				if (null != proId && !"".equals(proId)) {
					req.setProjectId(Integer.parseInt(this.getProjectId()));
				}
				CreateKeyPairResponse resp = client.CreateKeyPair(req);
				return new BocloudResult(true, convertor.convertModel(resp.getKeyPair()), "创建成功");
			}
		} catch (Exception e) {
			logger.error("创建失败:", e);
			return new BocloudResult(false, "创建失败！", e.getMessage());
		}
	}

	@Override
	public BocloudResult delete(String keypairName) {
		try {
			DeleteKeyPairsRequest request = new DeleteKeyPairsRequest();
			request.setKeyIds(new String[] { keypairName });
			CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
			DeleteKeyPairsResponse response = client.DeleteKeyPairs(request);
			return new BocloudResult(true, response.getRequestId(), "删除成功");
		} catch (TencentCloudSDKException e) {
			logger.error("KeyPair 删除 [{}] error:", e);
			return new BocloudResult(false, "删除失败！", e.getMessage());
		}
	}

	@Override
	public BocloudResult modify(KeypairModel model) {
		try {
			ModifyKeyPairAttributeRequest request = new ModifyKeyPairAttributeRequest();
			request.setKeyName(model.getName());
			request.setKeyId(model.getKeyId());
			CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
			ModifyKeyPairAttributeResponse keyPairAttribute = client.ModifyKeyPairAttribute(request);
			return new BocloudResult(true, keyPairAttribute.getRequestId(), "修改成功");
		} catch (TencentCloudSDKException e) {
			logger.error("KeyPair 修改 [{}] error:", e);
			return new BocloudResult(false, "修改失败！", e.getMessage());
		}
	}

}
