package com.bocloud.cmp.provider.storage;

import java.util.Arrays;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.bocloud.cmp.common.enums.TencentEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.model.h3c.H3cVolumeModel;
import com.bocloud.cmp.provider.TencentProvider;
import com.bocloud.cmp.provider.compute.TencentSnapshotProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.tencentconvertor.VolumeConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.tencentcloudapi.cbs.v20170312.CbsClient;
import com.tencentcloudapi.cbs.v20170312.models.AttachDisksRequest;
import com.tencentcloudapi.cbs.v20170312.models.AttachDisksResponse;
import com.tencentcloudapi.cbs.v20170312.models.CreateDisksRequest;
import com.tencentcloudapi.cbs.v20170312.models.CreateDisksResponse;
import com.tencentcloudapi.cbs.v20170312.models.CreateSnapshotRequest;
import com.tencentcloudapi.cbs.v20170312.models.CreateSnapshotResponse;
import com.tencentcloudapi.cbs.v20170312.models.DescribeDisksRequest;
import com.tencentcloudapi.cbs.v20170312.models.DescribeDisksResponse;
import com.tencentcloudapi.cbs.v20170312.models.DetachDisksRequest;
import com.tencentcloudapi.cbs.v20170312.models.DetachDisksResponse;
import com.tencentcloudapi.cbs.v20170312.models.Disk;
import com.tencentcloudapi.cbs.v20170312.models.DiskChargePrepaid;
import com.tencentcloudapi.cbs.v20170312.models.Filter;
import com.tencentcloudapi.cbs.v20170312.models.ModifyDiskAttributesRequest;
import com.tencentcloudapi.cbs.v20170312.models.ModifyDiskAttributesResponse;
import com.tencentcloudapi.cbs.v20170312.models.Placement;
import com.tencentcloudapi.cbs.v20170312.models.ResizeDiskRequest;
import com.tencentcloudapi.cbs.v20170312.models.ResizeDiskResponse;
import com.tencentcloudapi.cbs.v20170312.models.TerminateDisksRequest;
import com.tencentcloudapi.cbs.v20170312.models.TerminateDisksResponse;
import com.tencentcloudapi.common.exception.TencentCloudSDKException;

/**
 * 卷操作类
 * 
 * @author ymh
 * @时间 2018年11月15日
 */
@Provider("bocloud.cmp.tencent")
public class TencentVolumeProvider extends TencentProvider implements VolumeProvider {

	private static Logger logger = LoggerFactory.getLogger(TencentVolumeProvider.class);

	private BeanConvertor<VolumeModel, Disk> convertor = new VolumeConvertor();

	private Butler butler;

	public TencentVolumeProvider(Butler butler) {
		super(butler);
		this.butler = butler;
	}

	@Override
	public BocloudResult list() {
		try {
			DescribeDisksRequest req = new DescribeDisksRequest();
			CbsClient client = (CbsClient) this.getClient(TencentEnum.CBS);
			DescribeDisksResponse resp = client.DescribeDisks(req);
			Disk[] diskSet = resp.getDiskSet();
			return new BocloudResult(true, convertor.convertList(Arrays.asList(diskSet)), "查询成功");
		} catch (TencentCloudSDKException e) {
			return new BocloudResult(false, "查询失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult list(String instanceId) {
		try {
			DescribeDisksRequest req = new DescribeDisksRequest();
			Filter f = new Filter();
			f.setName("instance-id");
			f.setValues(new String[] { instanceId });
			Filter[] filters = new Filter[] { f };
			req.setFilters(filters);
			CbsClient client = (CbsClient) this.getClient(TencentEnum.CBS);
			DescribeDisksResponse resp = client.DescribeDisks(req);
			Disk[] diskSet = resp.getDiskSet();
			return new BocloudResult(true, convertor.convertList(Arrays.asList(diskSet)), "查询成功");
		} catch (TencentCloudSDKException e) {
			return new BocloudResult(false, "列表查询失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult create(VolumeModel volume) {
		try {
			CreateDisksRequest req = new CreateDisksRequest();
			// CLOUD_BASIC：表示普通云硬盘
			// CLOUD_PREMIUM：表示高性能云硬盘
			// CLOUD_SSD：表示SSD云硬盘。
			req.setDiskType(volume.getType());
			// PREPAID：预付费，即包年包月
			// POSTPAID_BY_HOUR：按小时后付费
			String diskChargeType = volume.getDiskChargeType();
			if (null == diskChargeType || "".equals(diskChargeType)) {
				diskChargeType = "POSTPAID_BY_HOUR";
			}
			req.setDiskChargeType(diskChargeType); // 按小时计费
			volume.setDiskChargeType(diskChargeType);
			if ("PREPAID".equalsIgnoreCase(diskChargeType)) {
				DiskChargePrepaid diskChargePrepaid = new DiskChargePrepaid();
				Integer period = volume.getPeriod();
				if (null == period || period <= 0) {
					period = 1;
				}
				diskChargePrepaid.setPeriod(period);
				req.setDiskChargePrepaid(diskChargePrepaid);
			}
			Placement placement = new Placement();
			String proId = this.getProjectId();
			if (null != proId && !"".equals(proId)) {
				placement.setProjectId(Integer.parseInt(this.getProjectId()));
			}
			placement.setZone(volume.getZone());
			req.setPlacement(placement);
			CbsClient client = (CbsClient) this.getClient(TencentEnum.CBS);
			if (null != volume.getSourceSnapshotId() && !"".equals(volume.getSourceSnapshotId())) {
				req.setSnapshotId(volume.getSourceSnapshotId());
			} else {
				req.setDiskSize(volume.getSize());
			}
			req.setDiskName(volume.getName());
			CreateDisksResponse createDisks = client.CreateDisks(req);
			String[] diskIdSet = createDisks.getDiskIdSet();
			if (null != diskIdSet && diskIdSet.length > 0) {
				BocloudResult detail = this.detail(diskIdSet[0]);
				VolumeModel convertModel = (VolumeModel) detail.getData();
				convertModel.setRegionId(this.getRegionId());
				return new BocloudResult(true, convertModel, "创建成功！");
			}
			return new BocloudResult(false, volume, "创建失败！");
		} catch (TencentCloudSDKException e) {
			return new BocloudResult(false, "创建失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult modify(VolumeModel volume) {
		try {
			ModifyDiskAttributesRequest req = new ModifyDiskAttributesRequest();
			req.setDiskName(volume.getName());
			req.setDeleteWithInstance(volume.getDeleteWithInstance());
			req.setDiskIds(new String[] { volume.getVolumeUuid() });
			CbsClient client = (CbsClient) this.getClient(TencentEnum.CBS);
			ModifyDiskAttributesResponse resp = client.ModifyDiskAttributes(req);
			return new BocloudResult(true, resp.getRequestId(), "修改成功");
		} catch (Exception e) {
			logger.error("修改失败 ： " + e);
			return new BocloudResult(false, "修改失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String volumeId) {
		try {
			TerminateDisksRequest req = new TerminateDisksRequest();
			req.setDiskIds(new String[] { volumeId });
			CbsClient client = (CbsClient) this.getClient(TencentEnum.CBS);
			TerminateDisksResponse resp = client.TerminateDisks(req);
			return new BocloudResult(true, resp.getRequestId(), "删除成功");
		} catch (Exception e) {
			logger.error("删除失败 ： " + e);
			return new BocloudResult(false, "删除失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String volumeId) {
		try {
			DescribeDisksRequest req = new DescribeDisksRequest();
			req.setDiskIds(new String[] { volumeId });
			CbsClient client = (CbsClient) this.getClient(TencentEnum.CBS);
			DescribeDisksResponse resp = client.DescribeDisks(req);
			Disk[] diskSet = resp.getDiskSet();
			VolumeModel convertModel = new VolumeModel();
			if (null != diskSet && diskSet.length > 0) {
				convertModel = convertor.convertModel(diskSet[0]);
			} else {
				try {
					Thread.sleep(2000);
				} catch (Exception e) {
					logger.error("线程异常：" + e.getMessage());
				}
				return this.detail(volumeId);
			}
			return new BocloudResult(true, convertModel, "查询成功");
		} catch (TencentCloudSDKException e) {
			return new BocloudResult(false, "查询失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult snapshot(SnapshotModel snapshotModel) {
		try {
			CreateSnapshotRequest req = new CreateSnapshotRequest();
			req.setDiskId(snapshotModel.getVolumeUuid());
			req.setSnapshotName(snapshotModel.getName());
			CbsClient client = (CbsClient) this.getClient(TencentEnum.CBS);
			CreateSnapshotResponse resp = client.CreateSnapshot(req);
			return new BocloudResult(true, this.snapshotDetail(resp.getSnapshotId()).getData(), "创建快照成功");
		} catch (Exception e) {
			logger.error("创建快照失败 : " + e);
			return new BocloudResult(false, "创建快照失败", e.getMessage());
		}
	}

	private BocloudResult snapshotDetail(String snapshotId) {
		TencentSnapshotProvider provider = new TencentSnapshotProvider(butler);
		SnapshotModel model = new SnapshotModel();
		model.setSnapshotUuid(snapshotId);
		BocloudResult detail = provider.detail(model);
		return detail;
	}

	@Override
	public BocloudResult attach(String serverId, String volumeId) {
		try {
			AttachDisksRequest req = new AttachDisksRequest();
			req.setDeleteWithInstance(false);
			req.setDiskIds(new String[] { volumeId });
			req.setInstanceId(serverId);
			CbsClient client = (CbsClient) this.getClient(TencentEnum.CBS);
			AttachDisksResponse resp = client.AttachDisks(req);
			return new BocloudResult(true, resp.getRequestId(), "挂载成功");
		} catch (Exception e) {
			logger.error("挂载失败: " + e);
			return new BocloudResult(false, "挂载失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult attach(String serverId, String mountpoint, String volumeId) {
		try {
			AttachDisksRequest req = new AttachDisksRequest();
			req.setDeleteWithInstance(false);
			req.setDiskIds(new String[] { volumeId });
			req.setInstanceId(serverId);
			CbsClient client = (CbsClient) this.getClient(TencentEnum.CBS);
			AttachDisksResponse resp = client.AttachDisks(req);
			return new BocloudResult(true, resp.getRequestId(), "挂载成功");
		} catch (Exception e) {
			return new BocloudResult(false, "挂载失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult detach(String serverId, String volumeId) {
		try {
			CbsClient client = (CbsClient) this.getClient(TencentEnum.CBS);
			DetachDisksRequest req = new DetachDisksRequest();
			req.setDiskIds(new String[] { volumeId });
			DetachDisksResponse resp = client.DetachDisks(req);
			return new BocloudResult(true, resp.getRequestId(), "卸载成功");
		} catch (Exception e) {
			return new BocloudResult(false, "卸载失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult resetStatus(String volumeId, String status) {
		return null;
	}

	@Override
	public BocloudResult uploadToImage(String volumeId, ImageModel imageModel) {
		return null;
	}

	@Override
	public BocloudResult extend(String volumeId, Integer newSize) {
		try {
			ResizeDiskRequest req = new ResizeDiskRequest();
			req.setDiskId(volumeId);
			req.setDiskSize(newSize);
			CbsClient client = (CbsClient) this.getClient(TencentEnum.CBS);
			ResizeDiskResponse resp = client.ResizeDisk(req);
			return new BocloudResult(true, resp.getRequestId(), "扩容成功");
		} catch (Exception e) {
			logger.error("扩容失败 ： " + e);
			return new BocloudResult(false, e.getMessage(), e.getMessage());
		}
	}

	@Override
	public BocloudResult list(String hostId, String storagePoolName) {
		return null;
	}

	@Override
	public BocloudResult remove(H3cVolumeModel arg0) {
		return null;
	}

}
