package com.bocloud.cmp.tencentconvertor;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.common.enums.TencentConstants;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import com.tencentcloudapi.cvm.v20170312.models.DataDisk;
import com.tencentcloudapi.cvm.v20170312.models.Instance;
import com.tencentcloudapi.cvm.v20170312.models.InternetAccessible;
import com.tencentcloudapi.cvm.v20170312.models.LoginSettings;
import com.tencentcloudapi.cvm.v20170312.models.VirtualPrivateCloud;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 虚拟机转换器
 */
public class ServerConvertor implements BeanConvertor<ServerModel, Instance> {

    private static Logger logger = LoggerFactory.getLogger(ServerConvertor.class);

    @Override
    public List<ServerModel> convertList(List<? extends Instance> objects) {
        List<ServerModel> serverModels = new ArrayList<>();
        for (Instance object : objects) {
            serverModels.add(convertModel(object));
        }
        return serverModels;
    }

    @Override
    public ServerModel convertModel(Instance instance) {
        ServerModel model = new ServerModel();
        if (null != instance) {
            try {
                model.setCpu(instance.getCPU());
                model.setMemory(BigDecimal.valueOf(instance.getMemory()));
                model.setFlavorUuid(instance.getInstanceType());
                model.setImageUuid(instance.getImageId());
                model.setInstanceId(instance.getInstanceId());
                String instanceName = instance.getInstanceName();
                if ("未命名".equals(instanceName)) {
                    instanceName = instance.getInstanceId();
                }
                model.setGmtVmCreate(DateTools.transfer(TencentConstants.DATE_FORMAT, instance.getCreatedTime(), "Z"));
				String expiredTime = instance.getExpiredTime();
				if (!StringUtils.isEmpty(expiredTime)) {
					model.setGmtExpiredTime(DateTools.transfer(TencentConstants.DATE_FORMAT, expiredTime, "Z"));
				}
                model.setName(instanceName);
                model.setStatus(instance.getInstanceState());
                String[] securityGroupIds = instance.getSecurityGroupIds();
                if (null != securityGroupIds && securityGroupIds.length > 0) {
                    model.setSecurityGroups(Arrays.asList(securityGroupIds));
                }
                model.setSystemDiskCategory(instance.getSystemDisk().getDiskType());
                model.setOsName(instance.getOsName());
                model.setZone(instance.getPlacement().getZone());
                model.setSystemDiskSize(instance.getSystemDisk().getDiskSize());
                model.setSystemDiskCategory(instance.getSystemDisk().getDiskType());
                model.setInstanceChargeType(instance.getInstanceChargeType());
                this.ipInfoConvert(instance, model);
                this.internetAccessibleConvert(instance, model);
                this.diskConvert(instance, model);
                this.keypairConvert(instance, model);
            } catch (Exception e) {
                logger.error("转换异常：" + e.getMessage());
            }
        }
        return model;
    }

    private ServerModel keypairConvert(Instance instance, ServerModel model) {
        LoginSettings loginSettings = instance.getLoginSettings();
        if (null != loginSettings) {
            if (null != loginSettings.getKeyIds() && loginSettings.getKeyIds().length > 0) {
                model.setKeypairName(JSON.toJSONString(loginSettings.getKeyIds()));
            }
        }
        return model;
    }

    private ServerModel ipInfoConvert(Instance instance, ServerModel model) {
        String[] privateIpAddresses = instance.getPrivateIpAddresses();
        if (null != privateIpAddresses && privateIpAddresses.length > 0) {
            model.setPrivateIps(JSON.toJSONString(privateIpAddresses));
        }
        String[] publicIpAddresses = instance.getPublicIpAddresses();
        if (null != publicIpAddresses && publicIpAddresses.length > 0) {
            model.setPublicIps(JSON.toJSONString(publicIpAddresses));
        }
        return model;
    }

    private ServerModel internetAccessibleConvert(Instance instance, ServerModel model) {
        InternetAccessible internetAccessible = instance.getInternetAccessible();
        if (null != internetAccessible) {
            model.setInternetMaxBandwidthOut(internetAccessible.getInternetMaxBandwidthOut());
            model.setInternetChargeType(internetAccessible.getInternetChargeType());
            model.setIsUsePublicIp(internetAccessible.getPublicIpAssigned());
        }

        VirtualPrivateCloud virtualPrivateCloud = instance.getVirtualPrivateCloud();
        if (null != virtualPrivateCloud) {
            model.setNetworkUuid(virtualPrivateCloud.getVpcId());
            model.setSubnetUuid(virtualPrivateCloud.getSubnetId());
        }
        return model;
    }

    private ServerModel diskConvert(Instance instance, ServerModel model) {
        List<VolumeModel> diskDevices = new ArrayList<>();
        VolumeModel volumeModel = new VolumeModel();
        DataDisk[] dataDisks = instance.getDataDisks();
        if (null != dataDisks && dataDisks.length > 0) {
            for (DataDisk disk : dataDisks) {
                volumeModel = new VolumeModel();
                volumeModel.setVolumeUuid(disk.getDiskId());
                volumeModel.setType(disk.getDiskType());
                volumeModel.setSize(disk.getDiskSize());
                diskDevices.add(volumeModel);
            }
        }
        model.setDiskDevices(diskDevices);
        return model;
    }
}
